@@ -46,6 +46,13 @@
 #endif
 
 #ifdef NEXTSTEP3
+/* not supported */
+#define SA_RESTART 0
+#define O_RSYNC 0
+/* select, ftruncate, fchmod */
+#include <libc.h>
+/* in ansi string.h but undefined in -posix */
+#define bzero(b,len) memset(b,0,len)
 #define SIZEOF_IS_LONG_INT
 #define HAVE_GETRUSAGE
 #define PATH_TMP        "/private/tmp"
@@ -66,16 +73,43 @@
 #define PATH_SYSLOG     "/var/adm/SYSLOG"
 #endif
 
+#ifdef UNIXWARE7
+#define HAVE_GETRUSAGE
+#define O_RSYNC		0
+#define PATH_TMP        "/tmp"
+#define PATH_VAR_TMP    "/var/tmp"
+#define PATH_PASSWD     "/etc/passwd"
+#define PATH_WTMP       "/var/adm/wtmp"
+#define PATH_UTMP       "/etc/utmp"
+#define PATH_SYSLOG     "/var/adm/syslog"
+#endif
+
 #ifndef RANDSAVENAME
 #define RANDSAVENAME	"/etc/prngd-seed"
 #endif
 #ifndef CONFIGFILE
 #define CONFIGFILE	"/etc/prngd.conf"
 #endif
-#ifndef RANDSAVELEN
-#define RANDSAVELEN	1024
+
+/*
+ * If not available, we cannot perform the check and will always succeed.
+ */
+#ifndef S_ISSOCK
+#define S_ISSOCK(m)	(1)
 #endif
 
+/*
+ * Some platforms don't have the SHUT_* macros
+ */
+#ifndef SHUT_RDWR
+#define SHUT_RDWR	2
+#endif
+
+/*
+ * The pool size of the OpenSSL PRNG
+ */
+#define PRNGD_STATE_SIZE	1024
+
 #ifndef SEED_STAT_INTERVAL
 #define SEED_STAT_INTERVAL	17
 #endif
