@@ -6,6 +6,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <time.h>
+#include <assert.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -17,14 +18,6 @@
 #include "config.h"
 #include "prngd.h"
 
-#ifdef NEXTSTEP3
-/* not supported */
-#define O_RSYNC 0
-
-/* ftruncate/fchmod */
-#include <libc.h>
-#endif
-
 int randsavefd;
 int debug = 0;
 
@@ -35,7 +28,7 @@ static void write_seedfile(void);
 static void open_seedfile(const char *seedfile)
 {
   int bytes, bytes_read_total;
-  unsigned char buffer[1024];
+  unsigned char buffer[PRNGD_STATE_SIZE];
 
   randsavefd = open(seedfile, O_RDWR | O_CREAT | O_SYNC | O_RSYNC, 400);
   if (randsavefd < 0)
@@ -57,13 +50,13 @@ static void open_seedfile(const char *seedfile)
   /*
    * Do some initial seeding stuff to pre-stir the PRNG.
    */
-  seed_internal();
+  seed_internal(NULL);
 
   /*
    * Now read as much from the file as we can get. 
    */
   bytes_read_total = 0;
-  while ((bytes = read(randsavefd, buffer, 1024)) > 0)
+  while ((bytes = read(randsavefd, buffer, PRNGD_STATE_SIZE)) > 0)
   {
     rand_add(buffer, bytes, (double)bytes);
     bytes_read_total += bytes;
@@ -77,7 +70,7 @@ static void open_seedfile(const char *seedfile)
   /*
    * Done, let's stir around some more
    */
-  seed_internal();
+  seed_internal(NULL);
 
   if (debug)
     fprintf(stdout, "Read %d bytes\n", bytes_read_total);
@@ -91,7 +84,7 @@ static void open_seedfile(const char *seedfile)
 static void write_seedfile(void)
 {
   int bytes;
-  unsigned char buffer[1024];
+  unsigned char buffer[PRNGD_STATE_SIZE];
 
   bytes = lseek(randsavefd, 0, SEEK_SET);
   if (bytes < 0)
@@ -99,14 +92,14 @@ static void write_seedfile(void)
     fprintf(stderr, "Error writing seedfile: %s\n", strerror(errno));
     exit(EXIT_FAILURE);
   }
-  bytes = rand_bytes(buffer, 1024);
+  bytes = rand_bytes(buffer, PRNGD_STATE_SIZE);
   if (bytes == 0)
   {
     fprintf(stderr, "Could not retrieve rand_bytes\n");
   }
   else
   {
-    bytes = write(randsavefd, buffer, 1024);
+    bytes = write(randsavefd, buffer, PRNGD_STATE_SIZE);
     if (bytes < 0)
     {
       fprintf(stderr, "Error writing seedfile: %s\n", strerror(errno));
@@ -172,7 +165,7 @@ static void killpeer(int service_socket)
 static void usage(const char *progname)
 {
   fprintf(stderr,
-	  "Usage: %s [options] /tmp/entropy\nOptions:\n\t-d: debugging on\n\t-c cmdpath: use cmdpath for entropy commands [%s]\n\t-s seedpath: use seedpath as seedfile [%s]\n\t--kill: kill daemon on other side with HUP\n",
+	  "Usage: %s [options] /path/to/socket1 [/path/to/socket2 ...]\nOptions:\n\t-d: debugging on\n\t-c cmdpath: use cmdpath for entropy commands [%s]\n\t-s seedpath: use seedpath as seedfile [%s]\n\t--kill: kill daemon on other side\n",
 	  progname, CONFIGFILE, RANDSAVENAME);
 }
 
@@ -181,14 +174,17 @@ int main(int argc, char *argv[])
   int argp;
   int ret;
   int flags;
+  int statval;
   int killmode = 0;
   time_t seed_stat_interval = SEED_STAT_INTERVAL;
   time_t seed_ext_interval = SEED_EXT_INTERVAL;
-  char *randsock = NULL;
+  int i, numsock = 0;
+  char **randsock = NULL;
   char *cmdpath = CONFIGFILE;
   char *seedpath = RANDSAVENAME;
-  int service_socket;
-  struct sockaddr_un sockun;
+  int *service_socket, kill_socket;
+  struct sockaddr_un sockun, sockun_kill;
+  struct stat randsock_stat;
   entropy_source_t *entropy_source;
 
   /*
@@ -205,8 +201,17 @@ int main(int argc, char *argv[])
       seedpath = argv[++argp];
     else if (!strcmp(argv[argp], "--kill"))
       killmode = 1;
-    else if ((argv[argp][0] != '-') && (randsock == NULL))
-      randsock = argv[argp];
+    else if (argv[argp][0] != '-') {
+      if (numsock == 0)
+        randsock = malloc(sizeof(char *));
+      else
+	randsock = realloc(randsock, (numsock + 1) * sizeof(char *));
+      if (randsock == NULL) {
+	fprintf(stderr, "Could not allocate memory: %s\n", strerror(errno));
+	exit(EXIT_FAILURE);
+      }
+      randsock[numsock++] = argv[argp];
+    }
     else
     {
       usage(argv[0]);
@@ -214,7 +219,7 @@ int main(int argc, char *argv[])
     }
     argp++;
   }
-  if (!randsock) {
+  if (!numsock) {
     usage(argv[0]);
     exit(EXIT_FAILURE);
   }
@@ -222,7 +227,7 @@ int main(int argc, char *argv[])
   if (!killmode)
     parse_configfile(cmdpath, &entropy_source);
 
-  if (!randsock)
+  if (!numsock)
   {
     fprintf(stderr, "No socket name given, exiting!\n");
     exit(EXIT_FAILURE);
@@ -245,87 +250,153 @@ int main(int argc, char *argv[])
   if (!killmode)
     open_seedfile(seedpath);
 
+  if (!(service_socket = malloc(numsock * sizeof(int)))) {
+    fprintf(stderr, "Could not allocate memory: %s\n", strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+
   /*
-   * Set up the socket to service entropy requests
+   * Prepare the socket used for killing or testing for a running
+   * daemon.
    */
-  service_socket = socket(AF_UNIX, SOCK_STREAM, 0);
-  if (service_socket < 0)
-  {
+  kill_socket = socket(AF_UNIX, SOCK_STREAM, 0);
+  if (kill_socket < 0) {
     fprintf(stderr, "Could not create socket: %s\n", strerror(errno));
     exit(EXIT_FAILURE);
   }
   /*
-   * Bind the socket to the actual socket-file
+   * Set up the sockets to service entropy requests
    */
-  sockun.sun_family = AF_UNIX;
-  if (sizeof(sockun.sun_path) < strlen(randsock) + 1)
-  {
+  for (i = 0; i < numsock; i++) {
+
+    /*
+     * Check out what is already available at the place of the future
+     * egd-socket. If we cannot reach the place to be with "stat()",
+     * do not continue, as something may be seriously wrong. If the
+     * entry does already exist, remove() it, but only if it is of type
+     * socket! If removal fails for whatever reason, stop, as something
+     * may be seriously wrong.
+     */
+    if ((statval = stat(randsock[i], &randsock_stat) < 0) &&
+	(errno != ENOENT)) {
+      fprintf(stderr, "Cannot stat socket position %s: %s\n", randsock[i],
+	      strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+    else if (!statval) {
+      if (!S_ISSOCK(randsock_stat.st_mode)) {
+	fprintf(stderr, "Will not touch %s: no socket\n", randsock[i]);
+	exit(EXIT_FAILURE);
+      } else {
+	/*
+	 * Now that we have found a socket, we must check whether there
+	 * is a daemon listining on it. When in kill mode, this is great
+	 * and we are going to kill it. When not in kill mode, we cannot
+	 * start a new daemon at this address and exit with error.
+	 */
+	memset(&sockun, 0, sizeof(sockun_kill));
+	sockun_kill.sun_family = AF_UNIX;
+	if (sizeof(sockun_kill.sun_path) < strlen(randsock[i]) + 1) {
 #ifdef SIZEOF_IS_LONG_INT
-    fprintf(stderr, "Sockfilename too long: %s, maxlen=%ld\n", randsock,
-	    sizeof(sockun.sun_path));
+	  fprintf(stderr, "Sockfilename too long: %s, maxlen=%ld\n",
+		  randsock[i], sizeof(sockun_kill.sun_path));
 #else
-    fprintf(stderr, "Sockfilename too long: %s, maxlen=%d\n", randsock,
-	    sizeof(sockun.sun_path));
+	  fprintf(stderr, "Sockfilename too long: %s, maxlen=%d\n",
+		  randsock[i], sizeof(sockun_kill.sun_path));
 #endif /* SIZEOF_IS_LONG_INT */
-    exit(EXIT_FAILURE);
-  }
+	  exit(EXIT_FAILURE);
+        }
 
-  strcpy(sockun.sun_path, randsock);
+	strcpy(sockun_kill.sun_path, randsock[i]);
+	if (connect(kill_socket, (struct sockaddr *) &sockun_kill,
+		    sizeof(sockun_kill)) < 0) {
+	  if (killmode) {
+	    fprintf(stderr, "Cannot kill peer at %s: no daemon\n", randsock[i]);
+	    /*
+	     * Do no exit because there may be more work to do!
+	     */
+	    continue;
+	  }
+	}
+	else if (killmode) {
+	  killpeer(kill_socket);
+	  shutdown(kill_socket, SHUT_RDWR);
+	  continue;
+	}
+	else {
+	  fprintf(stderr,
+		  "Socket %s already used by another process, exiting.\n",
+		  randsock[i]);
+	  exit(EXIT_FAILURE);
+	}
+	shutdown(kill_socket, SHUT_RDWR);
 
-  /*
-   * Check, whether there already is a server running at this socket.
-   * In this case: don't touch the socket.
-   */
-  if (connect(service_socket, (struct sockaddr *) &sockun,
-	      sizeof(struct sockaddr_un)) < 0)
-  {
-    if (killmode)
+	/*
+         * Now there is a socket without service listening on it: we can
+	 * remove it.
+	 */
+	if (remove(randsock[i]) < 0) {
+	  fprintf(stderr, "Cannot remove already existing entry %s: %s\n",
+		  randsock[i], strerror(errno));
+	  exit(EXIT_FAILURE);
+	}
+      }
+    }
+
+    /*
+     * We can never reach this point in killmode, as for whatever condition
+     * not forcing immediate abort the loop was "continue"ed before.
+     */
+    assert(!killmode);
+
+    service_socket[i] = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (service_socket[i] < 0)
     {
-      fprintf(stderr, "Cannot kill peer at %s: %d\n", randsock,
-	      service_socket);
+      fprintf(stderr, "Could not create socket: %s\n", strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+    /*
+     * Bind the socket to the actual socket-file
+     */
+    memset(&sockun, 0, sizeof(sockun));
+    sockun.sun_family = AF_UNIX;
+    strcpy(sockun.sun_path, randsock[i]);
+    if (bind(service_socket[i], (struct sockaddr *) &sockun,
+	     sizeof(sockun)) < 0)
+    {
+      fprintf(stderr, "Could not bind socket to %s: %s\n", randsock[i],
+	      strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+    if (listen(service_socket[i], SOMAXCONN) < 0)
+    {
+      fprintf(stderr, "Could not listen on socket: %s\n", strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+    /*
+     * Switch the service_socket to non-blocking mode (otherwise it might
+     * hang inside accept() if a connection is closed between select() and
+     * accept().
+     */
+    if ((flags = fcntl(service_socket[i], F_GETFL, 0)) < 0)
+    {
+      fprintf(stderr, "Could not get socket file mode: %s\n",
+	      strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+    if (fcntl(service_socket[i], F_SETFL, flags | O_NONBLOCK) < 0)
+    {
+      fprintf(stderr, "Could not set socket file mode: %s\n",
+	      strerror(errno));
       exit(EXIT_FAILURE);
     }
   }
-  else if (killmode)
-  {
-    killpeer(service_socket);
-  }
-  else
-  {
-    fprintf(stderr, "Socket %s already used by another process: %s\n",
-	    randsock, strerror(errno));
-    exit(EXIT_FAILURE);
-  }
 
-  remove(randsock);
-  if (bind(service_socket, (struct sockaddr *) &sockun,
-	   sizeof(struct sockaddr_un)) < 0)
-  {
-    fprintf(stderr, "Could not bind socket to %s: %s\n", randsock,
-	    strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-  if (listen(service_socket, SOMAXCONN) < 0)
-  {
-    fprintf(stderr, "Could not listen on socket: %s\n", strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-  /*
-   * Switch the service_socket to non-blocking mode (otherwise it might
-   * hang inside accept() if a connection is closed between select() and
-   * accept().
-   */
-  if ((flags = fcntl(service_socket, F_GETFL, 0)) < 0)
-  {
-    fprintf(stderr, "Could not get socket file mode: %s\n",
-	    strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-  if (fcntl(service_socket, F_SETFL, flags | O_NONBLOCK) < 0)
-  {
-    fprintf(stderr, "Could not set socket file mode: %s\n",
-	    strerror(errno));
-    exit(EXIT_FAILURE);
+  if (killmode) {
+    /*
+     * Everything has been done!
+     */
+    exit(EXIT_SUCCESS);
   }
 
   /*
@@ -344,11 +415,11 @@ int main(int argc, char *argv[])
   /*
    * Maybe we have changed our pid, let's add some more of it. 
    */
-  seed_internal();
+  seed_internal(NULL);
 
   /* Never returns */
-  main_loop(service_socket, sockun, seed_stat_interval, seed_ext_interval,
-	    entropy_source, MAX_GATHERER_BYTES);
+  main_loop(service_socket, numsock, sockun, seed_stat_interval,
+	    seed_ext_interval, entropy_source, MAX_GATHERER_BYTES);
 
   /*
    * We never get here, but want to make the compiler happy...
