@@ -1,5 +1,10 @@
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <sys/time.h>
+#include <sys/times.h>
+#ifdef HAVE_GETRUSAGE
+#include <sys/resource.h>
+#endif
 
 #ifndef HAVE_DAEMON
 int daemon(int nochdir, int noclose);
@@ -14,19 +19,28 @@ typedef struct {
   char *cmdstring;
 } entropy_source_t;
 
+typedef struct {
+  struct tms tmsbuf;
+  struct timeval tp;
+  pid_t pid;
+#ifdef HAVE_GETRUSAGE
+  struct rusage usage;
+#endif
+} seed_t;
+
 extern int randsavefd;
 extern int debug;
 extern int devnull;
 
 void parse_configfile(const char *cmdpath, entropy_source_t **entropy_source);
 
-void main_loop(int service_socket, struct sockaddr_un sockun,
+void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	       time_t seed_stat_interval, time_t seed_ext_interval,
 	       entropy_source_t *entropy_source, int max_gatherer_bytes);
 
 void close_seedfile(void);
 
-void seed_internal(void);
+void seed_internal(seed_t *seed_p);
 void seed_stat(void);
 
 void rand_add(void *buf, int num, double entropy);
