@@ -13,23 +13,13 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <netinet/in.h>
-#include <openssl/rand.h>
 
 #include "config.h"
 #include "prngd.h"
 
-#ifdef NEXTSTEP3
-/* not supported */
-#define SA_RESTART 0
-
-/* select */
-#include <libc.h>
-
-/* in ansi string.h but undefined in -posix */
-# define bzero(b,len) memset(b,0,len)
-#endif
-
+static seed_t gatherer_seed;
 static pid_t gatherer_pid = -1;
+static int gatherer_status;
 
 typedef enum
 {
@@ -60,26 +50,33 @@ static void termination_handler(int received_signal)
 
 static void child_handler(int received_signal)
 {
-  int status;
   pid_t pid;
 
   if (debug)
     fprintf(stderr, "Caught SIGCHLD\n");
 
-  if ((pid = waitpid(-1, &status, WNOHANG)) == -1)
+  if ((pid = waitpid(-1, &gatherer_status, WNOHANG)) == -1)
 #ifdef PID_T_IS_LONG_INT
     fprintf(stderr, "Couldn't wait for child %ld: %s\n", pid, strerror(errno));
 #else
     fprintf(stderr, "Couldn't wait for child %d: %s\n", pid, strerror(errno));
 #endif
-  else if ((pid == gatherer_pid) && WIFEXITED(status))
+  else if ((pid == gatherer_pid) && WIFEXITED(gatherer_status))
      gatherer_pid = -1;
 
-  rand_add(&status, sizeof(status), 0.0);
-  seed_internal();
+  /*
+   * We should add the gatherer_status to the PRNG pool at this point, but
+   * we must not add entropy now because another rand_add() operation may be
+   * running just now. We hence save the data and add it later synchronized
+   * in the main_loop(). We don't need to care about the contents of
+   * gatherer_status et al and what happens if the connection is shut down
+   * from PRNGD because too many bytes were sent, since we don't care about
+   * the data at all, we just want to add to the pool as much as we can.
+   */
+  seed_internal(&gatherer_seed);
 }
 
-void main_loop(int service_socket, struct sockaddr_un sockun,
+void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 		      time_t seed_stat_interval, time_t seed_ext_interval,
 		      entropy_source_t *entropy_source, int max_gatherer_bytes)
 {
@@ -141,8 +138,13 @@ void main_loop(int service_socket, struct sockaddr_un sockun,
     FD_ZERO(&rfds);
     FD_ZERO(&wfds);
     FD_ZERO(&efds);
-    FD_SET(service_socket, &rfds);
-    max_fd = service_socket;
+    max_fd = 0;
+    for (i = 0; i < numsock; i++)
+    {
+      FD_SET(service_socket[i], &rfds);
+      if (service_socket[i] > max_fd)
+	max_fd = service_socket[i];
+    }
     if (gather_fd != -1)
     {
       FD_SET(gather_fd, &rfds);
@@ -187,6 +189,8 @@ void main_loop(int service_socket, struct sockaddr_un sockun,
 
     if (gather_fd != -1) {
       if (FD_ISSET(gather_fd, &efds)) {
+	if (debug)
+	  fprintf(stderr, "Closing connection to gatherer (efds)\n");
 	close(gather_fd);
 	gather_fd = -1;
       }
@@ -204,6 +208,8 @@ void main_loop(int service_socket, struct sockaddr_un sockun,
 	  if (debug)
 	    fprintf(stderr, "Closing connection to gatherer\n");
 	  close(gather_fd);
+	  rand_add(&gatherer_status, sizeof(gatherer_status), 0.0);
+	  rand_add(&gatherer_seed, sizeof(seed_t), 0.25);
 	  gather_fd = -1;
 	}
       }
@@ -476,63 +482,66 @@ void main_loop(int service_socket, struct sockaddr_un sockun,
       }
     }
 
-    if (FD_ISSET(service_socket, &rfds))
+    for (i = 0; i < numsock; i++)
     {
-      sockunlen = sizeof(sockun);
-      accept_fd = accept(service_socket, (struct sockaddr *) &sockun,
-			 &sockunlen);
-      if (accept_fd < 0)
-	continue;		/*
+      if (FD_ISSET(service_socket[i], &rfds))
+      {
+        sockunlen = sizeof(sockun);
+        accept_fd = accept(service_socket[i], (struct sockaddr *) &sockun,
+			   &sockunlen);
+        if (accept_fd < 0)
+	  continue;		/*
 				 * Ignore and stay alive 
 				 */
 
-      if ((flags = fcntl(accept_fd, F_GETFL, 0)) < 0)
-      {
-	if (debug)
-	  fprintf(stderr, "Could not get socket file mode: %s\n",
-		  strerror(errno));
-	close(accept_fd);
-	continue;
-      }
-      if (fcntl(accept_fd, F_SETFL, flags | O_NONBLOCK) < 0)
-      {
-	if (debug)
-	  fprintf(stderr, "Could not set socket file mode: %s\n",
-		  strerror(errno));
-	close(accept_fd);
-	continue;
-      }
+        if ((flags = fcntl(accept_fd, F_GETFL, 0)) < 0)
+        {
+	  if (debug)
+	    fprintf(stderr, "Could not get socket file mode: %s\n",
+		    strerror(errno));
+	  close(accept_fd);
+	  continue;
+        }
+        if (fcntl(accept_fd, F_SETFL, flags | O_NONBLOCK) < 0)
+        {
+	  if (debug)
+	    fprintf(stderr, "Could not set socket file mode: %s\n",
+		    strerror(errno));
+	  close(accept_fd);
+	  continue;
+        }
 
-      /*
-       * Create a new structure for the incoming request 
-       */
-      if (max_query + 1 > alloc_query)
-      {
-	egd_query_old = egd_query;
-	if (alloc_query == 0)
-	  egd_query = malloc((max_query + 1) * sizeof(egd_query_t));
-	else
-	  egd_query =
-	      realloc(egd_query_old,
-		      (max_query + 1) * sizeof(egd_query_t));
-	if (!egd_query)
-	{
-	  close(accept_fd);	/*
+        /*
+         * Create a new structure for the incoming request 
+         */
+        if (max_query + 1 > alloc_query)
+        {
+	  egd_query_old = egd_query;
+	  if (alloc_query == 0)
+	    egd_query = malloc((max_query + 1) * sizeof(egd_query_t));
+	  else
+	    egd_query =
+	        realloc(egd_query_old,
+		        (max_query + 1) * sizeof(egd_query_t));
+	  if (!egd_query)
+	  {
+	    close(accept_fd);	/*
 				 * Better this than complete shut down 
 				 */
-	  egd_query = egd_query_old;
-	  continue;
-	}
-	alloc_query = max_query + 1;
+	    egd_query = egd_query_old;
+	    continue;
+	  }
+	  alloc_query = max_query + 1;
+        }
+        /*
+         * This routine is safe here, as in case of error we do "continue;"
+         * in case of problems. So we have succesfully reached this point.
+         */
+        max_query++;
+        egd_query[max_query - 1].fd = accept_fd;
+        egd_query[max_query - 1].state = wait_command;
+        fprintf(stderr, "Opening service fd %d\n", egd_query[max_query-1].fd);
       }
-      /*
-       * This routine is safe here, as in case of error we do "continue;"
-       * in case of problems. So we have succesfully reached this point.
-       */
-      max_query++;
-      egd_query[max_query - 1].fd = accept_fd;
-      egd_query[max_query - 1].state = wait_command;
-      fprintf(stderr, "Opening service fd %d\n", egd_query[max_query-1].fd);
     }
 
   }
